import * as i0 from '@angular/core';
import { inject, ChangeDetectorRef, Directive, Injector, ErrorHandler, EnvironmentInjector, createEnvironmentInjector } from '@angular/core';
import { noop, isEqual, omit, keys, size, flow, bindKey } from '@s-libs/micro-dash';
import { isTruthy, wrapMethod } from '@s-libs/js-core';
import { mixInSubscriptionManager, delayOnMicrotaskQueue } from '@s-libs/rxjs-core';
import { Subject, BehaviorSubject, merge, of, tap, retry } from 'rxjs';
import { filter, distinctUntilChanged, map } from 'rxjs/operators';
import { NG_VALUE_ACCESSOR, NgControl, FormControl } from '@angular/forms';

/**
 * Mixes in {@link InjectableSuperclass} as an additional superclass.
 *
 * ```ts
 * class MySubclass extends mixInInjectableSuperclass(MyOtherSuperclass) {
 *   subscribeWithAutoUnsubscribe(observable: Observable<any>) {
 *     this.subscribeTo(observable);
 *   }
 * }
 * ```
 */
// eslint-disable-next-line @typescript-eslint/explicit-function-return-type,@typescript-eslint/explicit-module-boundary-types -- I'm not sure how to describe the return type
function mixInInjectableSuperclass(Base) {
    return class extends mixInSubscriptionManager(Base) {
        constructor() {
            super(...arguments);
            this.#destructionSubject = new Subject();
            /**
             * An observable that emits once when this object is destroyed, then completes.
             */
            // eslint-disable-next-line @typescript-eslint/member-ordering -- ordered like this to avoid writing a constructor, which causes a different error
            this.destruction$ = this.#destructionSubject.asObservable();
        }
        #destructionSubject;
        ngOnDestroy() {
            this.unsubscribe();
            this.#destructionSubject.next();
            this.#destructionSubject.complete();
        }
    };
}
/**
 * Use as the superclass for anything managed by angular's dependency injection for care-free use of `subscribeTo()`. It simply calls `unsubscribe()` during `ngOnDestroy()`. If you override `ngOnDestroy()` in your subclass, be sure to invoke the super implementation.
 *
 * ```ts
 * @Injectable() // or @Component(), @Directive() or @Pipe(), but consider DirectiveSuperclass
 * class MyThing extends InjectableSuperclass {
 *   constructor(somethingObservable: Observable) {
 *     super();
 *     this.subscribeTo(somethingObservable);
 *   }
 *
 *   ngOnDestroy() {
 *     // if you override ngOnDestroy, be sure to call this too
 *     super.ngOnDestroy();
 *   }
 * }
 * ```
 */
class InjectableSuperclass extends mixInInjectableSuperclass(Object) {
}

/**
 * Extend this when creating a directive (including a component, which is a kind of directive) to gain access to the helpers demonstrated below.
 *
 * ```ts
 * @Component({
 *   selector: "s-color-text",
 *   template: `
 *     <span [style.background]="color">{{ color }}</span>
 *   `,
 *   // note that `bindToInstance()` works even with OnPush change detection
 *   changeDetection: ChangeDetectionStrategy.OnPush,
 * })
 *  class ColorTextComponent extends DirectiveSuperclass {
 *   @Input() prefix?: string;
 *   @Input() prefix2?: string;
 *   color!: string;
 *
 *   constructor(@Inject("color$") color$: Observable<string>) {
 *     super();
 *
 *     // combine everything to calculate `color` and keep it up to date
 *     this.bindToInstance(
 *       "color",
 *       combineLatest(
 *         this.getInput$("prefix"),
 *         this.getInput$("prefix2"),
 *         color$,
 *       ).pipe(map((parts) => parts.filter((p) => p).join(""))),
 *     );
 *   }
 * }
 * ```
 */
// maybe this won't need the fake selector after https://github.com/angular/angular/issues/36427
class DirectiveSuperclass extends InjectableSuperclass {
    constructor() {
        super(...arguments);
        /**
         *  Emits the set of `@Input()` property names that change during each call to `ngOnChanges()`.
         */
        this.inputChanges$ = new Subject();
        this.#changeDetectorRef = inject(ChangeDetectorRef);
        this.#onChangesRan$ = new BehaviorSubject(false);
    }
    #changeDetectorRef;
    #onChangesRan$;
    ngOnChanges(changes) {
        this.inputChanges$.next(new Set(Object.getOwnPropertyNames(changes)));
        this.#onChangesRan$.next(true);
    }
    /**
     * @return an observable of the values for one of this directive's `@Input()` properties
     */
    getInput$(key) {
        // Should emit:
        //   - immediately if ngOnChanges was already called
        //   - on the first call to ngOnChanges
        //   - after a delay if ngOnChanges is never called (when nothing is bound to the directive)
        //   - when the value actually changes
        return merge(this.#onChangesRan$.pipe(filter(isTruthy), distinctUntilChanged()), this.inputChanges$, of(0).pipe(delayOnMicrotaskQueue())).pipe(map(() => this[key]), distinctUntilChanged());
    }
    /**
     * Binds an observable to one of this directive's instance variables. When the observable emits the instance variable will be updated, and change detection will be triggered to propagate any changes. Use this as an alternative to repeating `| async` multiple times in your template.
     */
    bindToInstance(key, value$) {
        this.subscribeTo(value$, (value) => {
            this[key] = value;
            this.#changeDetectorRef.markForCheck();
        });
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "16.0.0", ngImport: i0, type: DirectiveSuperclass, deps: null, target: i0.ɵɵFactoryTarget.Directive }); }
    static { this.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "14.0.0", version: "16.0.0", type: DirectiveSuperclass, selector: "[slDirectiveSuperclass]", usesInheritance: true, usesOnChanges: true, ngImport: i0 }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "16.0.0", ngImport: i0, type: DirectiveSuperclass, decorators: [{
            type: Directive,
            args: [{ selector: '[slDirectiveSuperclass]' }]
        }] });

/**
 * Extend this when creating a form control to reduce some boilerplate.
 *
 * This example allows 2-way binding to a number via `[(ngModel)]`, `[formControl]`, or any other technique that leverages the `ControlValueAccessor` interface.
 * ```ts
 * @Component({
 *   template: `
 *     <button (click)="increment()" [disabled]="isDisabled">{{ counter }}</button>
 *   `,
 *   providers: [provideValueAccessor(CounterComponent)],
 * })
 * class CounterComponent extends FormComponentSuperclass<number> {
 *   counter = 0;
 *
 *   handleIncomingValue(value: number) {
 *     this.counter = value;
 *   }
 *
 *   increment() {
 *     this.emitOutgoingValue(++this.counter);
 *     this.onTouched();
 *   }
 * }
 * ```
 */
class FormComponentSuperclass extends DirectiveSuperclass {
    constructor() {
        super(...arguments);
        /** Call this to emit a new value when it changes. */
        this.emitOutgoingValue = noop;
        /** Call this to "commit" a change, traditionally done e.g. on blur. */
        this.onTouched = noop;
        /** You can bind to this in your template as needed. */
        this.isDisabled = false;
        this.#changeDetectorRef = inject(ChangeDetectorRef);
    }
    #changeDetectorRef;
    /** Called as angular propagates value changes to this `ControlValueAccessor`. You normally do not need to use it. */
    writeValue(value) {
        this.handleIncomingValue(value);
        this.#changeDetectorRef.markForCheck();
    }
    /** Called as angular sets up the binding to this `ControlValueAccessor`. You normally do not need to use it. */
    registerOnChange(fn) {
        this.emitOutgoingValue = fn;
    }
    /** Called as angular sets up the binding to this `ControlValueAccessor`. You normally do not need to use it. */
    registerOnTouched(fn) {
        this.onTouched = fn;
    }
    /** Called as angular propagates disabled changes to this `ControlValueAccessor`. You normally do not need to use it. */
    setDisabledState(isDisabled) {
        this.isDisabled = isDisabled;
        this.#changeDetectorRef.markForCheck();
    }
}

/**
 * Use in the `providers` of a component that implements `ControlValueAccessor` to reduce some boilerplate.
 *
 * ```ts
 * @Component({ providers: [provideValueAccessor(MyFormControl)] }
 * class MyFormControl extends BaseFormControl {
 *   // ...
 * }
 * ```
 */
function provideValueAccessor(type) {
    return {
        provide: NG_VALUE_ACCESSOR,
        useExisting: type,
        multi: true,
    };
}

class ControlSynchronizer {
    #errorsFromPartner = {};
    #originalSetErrors;
    #finishingAsync = false;
    constructor(control) {
        this.control = control;
        this.#originalSetErrors = this.control.setErrors.bind(this.control);
        wrapMethod(control, 'updateValueAndValidity', {
            around: (original, ...args) => {
                // when finishing async validators we instructed our partner to re-run its own validators. That could in turn try to re-trigger ours, dropping the errors we just set, putting us into a pending state, and re-triggering our own async validators that just completed.
                if (!this.#finishingAsync) {
                    original.apply(control, args);
                }
            },
        });
        wrapMethod(control, 'setErrors', {
            // ignore the second "opts" parameter that could set `emitEvent: false`. When async validation completes, we need to ensure a `statusChange` fires so we synchronize with `#partner`
            around: (_, newNativeErrors) => {
                this.#finishingAsync = true;
                this.#updateCombinedErrors(newNativeErrors);
                this.#finishingAsync = false;
            },
        });
        control.addValidators(() => this.#errorsFromPartner);
    }
    // eslint-disable-next-line max-params
    static synchronize(outer, inner, outerToInnerTransform, innerToOuterTransform, subscriptionManager) {
        const outerSync = new ControlSynchronizer(outer);
        const innerSync = new ControlSynchronizer(inner);
        outerSync.#acceptErrorsFrom(innerSync, innerToOuterTransform, subscriptionManager);
        innerSync.#acceptErrorsFrom(outerSync, outerToInnerTransform, subscriptionManager);
    }
    #acceptErrorsFrom(partner, transform, subscriptionManager) {
        subscriptionManager.subscribeTo(merge(partner.control.statusChanges, Promise.resolve()).pipe(map(() => partner.#getNativeErrors()), distinctUntilChanged(isEqual), transform), (transformedErrors) => {
            this.#setAddedErrors(transformedErrors);
        });
    }
    #setAddedErrors(errorsToAdd) {
        this.#errorsFromPartner = errorsToAdd;
        this.control.updateValueAndValidity({ emitEvent: false });
    }
    #getNativeErrors() {
        return omit(this.control.errors, ...keys(this.#errorsFromPartner));
    }
    #updateCombinedErrors(nativeErrors) {
        const combinedErrors = { ...nativeErrors, ...this.#errorsFromPartner };
        this.#originalSetErrors(size(combinedErrors) ? combinedErrors : null);
    }
}

/**
 * Extend this when creating a form component that simply wraps existing ones, to reduce a lot of boilerplate.
 *
 * The most common case is to use a simple {@linkcode FormControl}:
 * ```ts
 * @Component({
 *   template: `<input [formControl]="control" />`,
 *   providers: [provideValueAccessor(StringComponent)],
 * })
 * class StringComponent extends WrappedControlSuperclass<string | null> {
 *   control = new FormControl('');
 * }
 * ```
 *
 * Example when you need to modify the wrapped control's value:
 * ```ts
 * @Component({
 *   selector: 'sl-date',
 *   template: `<input type="datetime-local" [formControl]="control" />`,
 *   providers: [provideValueAccessor(DateComponent)],
 * })
 * class DateComponent extends WrappedControlSuperclass<
 *   Date | null,
 *   string | null
 * > {
 *   control = new FormControl<string | null>(null);
 *
 *   protected override innerToOuterValue(
 *     inner: string | null,
 *   ): Date | null {
 *     return inner ? new Date(`${inner}Z`) : null;
 *   }
 *
 *   protected override outerToInnerValue(
 *     outer: Date | null,
 *   ): string | null {
 *     return outer ? outer.toISOString().substring(0, 16) : null;
 *   }
 * }
 * ```
 *
 * Example of wrapping multiple inner components:
 * ```ts
 * class FullName {
 *   firstName = '';
 *   lastName = '';
 * }
 *
 * @Component({
 *   selector: 'sl-full-name',
 *   template: `
 *     <div [formGroup]="control">
 *       <input id="first" formControlName="firstName" />
 *       <input id="last" formControlName="lastName" />
 *     </div>
 *   `,
 *   providers: [provideValueAccessor(FullNameComponent)],
 * })
 * class FullNameComponent extends WrappedControlSuperclass<
 *   FullName | null,
 *   Partial<FullName>
 * > {
 *   control = new FormGroup({
 *     firstName: new FormControl('', { nonNullable: true }),
 *     lastName: new FormControl('', { nonNullable: true }),
 *   });
 *
 *   protected override outerToInnerValue(outer: FullName | null): FullName {
 *     // `formControlName` binding can't handle a null value
 *     return outer ?? new FullName();
 *   }
 *
 *   protected override innerToOuterValue(
 *     inner: Partial<FullName>,
 *   ): FullName {
 *     // the values in a `FormGroup` can be `undefined` when their corresponding controls are disabled
 *     return {
 *       firstName: inner.firstName ?? '',
 *       lastName: inner.lastName ?? '',
 *     };
 *   }
 * }
 * ```
 *
 * If you bind to your component using an {@linkcode NgControl} (e.g. when using `ngModel`), validation errors will be synchronized between it and the control inside your component. You can override various methods below to control or disable that process. Note that validation, `statuschanges`, and `valuechanges` may all happen more often as a result of this synchronization.
 */
class WrappedControlSuperclass extends FormComponentSuperclass {
    #incomingValues$ = new Subject();
    #injector = inject(Injector);
    #errorHandler = inject(ErrorHandler);
    constructor() {
        super();
        this.subscribeTo(this.setUpOuterToInnerValue$(this.#incomingValues$), (inner) => {
            this.control.setValue(inner, { emitEvent: false });
        });
    }
    ngOnInit() {
        // eslint-disable-next-line @typescript-eslint/no-floating-promises
        this.#bindValidation();
        this.subscribeTo(this.setUpInnerToOuterValue$(this.control.valueChanges), (outer) => {
            this.emitOutgoingValue(outer);
        });
        wrapMethod(this.control, 'markAsTouched', {
            after: () => {
                this.onTouched();
            },
        });
    }
    /** Called as angular propagates values changes to this `ControlValueAccessor`. You normally do not need to use it. */
    handleIncomingValue(outer) {
        this.#incomingValues$.next(outer);
    }
    /** Called as angular propagates disabled changes to this `ControlValueAccessor`. You normally do not need to use it. */
    setDisabledState(isDisabled) {
        if (isDisabled) {
            this.control.disable({ emitEvent: false });
        }
        else {
            this.control.enable({ emitEvent: false });
        }
        super.setDisabledState(this.isDisabled);
    }
    /**
     * Override this to for full control over the stream of values passed in to your subclass.
     *
     * In this example, incoming values are debounced before being passed through to the inner form control
     * ```ts
     * setUpOuterToInnerValue$(outer$: Observable<OuterType>): Observable<InnerType> {
     *   return outer$.pipe(
     *     debounce(300),
     *     map((outer) => doExpensiveTransformToInnerValue(outer)),
     *   );
     * }
     * ```
     *
     * For a simple transformation, see {@linkcode #outerToInnerValue} instead.
     */
    setUpOuterToInnerValue$(outer$) {
        return outer$.pipe(map((outer) => this.outerToInnerValue(outer)), this.#handleError());
    }
    /**
     * Override this to modify a value coming from the outside to the format needed within this component.
     *
     * For more complex needs, see {@linkcode #setUpOuterToInnerValue$} instead.
     */
    outerToInnerValue(outer) {
        return outer;
    }
    /**
     * Override this to for full control over the stream of values emitted from your subclass.
     *
     * In this example, illegal values are not emitted
     * ```ts
     * setUpInnerToOuterValue$(inner$: Observable<InnerType>): Observable<OuterType> {
     *   return inner$.pipe(
     *     filter((inner) => isLegalValue(outer)),
     *   );
     * }
     * ```
     *
     * For a simple transformation, see {@linkcode #innerToOuterValue} instead.
     */
    setUpInnerToOuterValue$(inner$) {
        return inner$.pipe(map((inner) => this.innerToOuterValue(inner)), this.#handleError());
    }
    /**
     * Override this to modify a value coming from within this component to the format expected on the outside.
     *
     * For more complex needs, see {@linkcode #setUpInnerToOuterValue$} instead.
     */
    innerToOuterValue(inner) {
        return inner;
    }
    /**
     * Override this to for full control over the stream of validation errors synchronized in to your subclass.
     *
     * For a simple transformation, see {@linkcode #outerToInnerErrors} instead.
     */
    setUpOuterToInnerErrors$(outer$) {
        return outer$.pipe(map((inner) => this.outerToInnerErrors(inner)), this.#handleError());
    }
    /**
     * Override this to modify validation errors that synchronize in to this component.
     *
     * In this example we assume the `required` validation is handled by the user and should not affect internal validation
     * ```ts
     * protected override outerToInnerErrors(
     *   errors: ValidationErrors,
     * ): ValidationErrors {
     *   return omit(errors, 'required');
     * }
     * ```
     *
     * For more complex needs, see {@linkcode #setUpOuterToInnerErrors$} instead.
     */
    outerToInnerErrors(errors) {
        return errors;
    }
    /**
     * Override this to for full control over the stream of validation errors synchronized out from your subclass.
     *
     * In this example, synchronization is turned off:
     * ```ts
     * protected override setUpInnerToOuterErrors$(): Observable<ValidationErrors> {
     *   return EMPTY;
     * }
     * ```
     *
     * For a simple transformation, see {@linkcode #innerToOuterErrors} instead.
     */
    setUpInnerToOuterErrors$(inner$) {
        return inner$.pipe(map((inner) => this.innerToOuterErrors(inner)), this.#handleError());
    }
    /**
     * Override this to modify validation errors that synchronize out from this component.
     *
     * For more complex needs, see {@linkcode #setUpInnerToOuterErrors$} instead.
     */
    innerToOuterErrors(errors) {
        return errors;
    }
    async #bindValidation() {
        // Hack-fixing a production bug: https://github.com/simontonsoftware/s-libs/issues/82. `ngModel` and `formControl` both set their `.control` before this component is initialized. However, `formControlName` does not. This is a timing hack to accommodate by delaying on the microtask queue.
        await Promise.resolve();
        const outerControl = this.#selfInject(NgControl)?.control;
        if (outerControl) {
            ControlSynchronizer.synchronize(outerControl, this.control, this.setUpOuterToInnerErrors$.bind(this), this.setUpInnerToOuterErrors$.bind(this), this);
        }
    }
    #handleError() {
        return flow(tap({ error: bindKey(this.#errorHandler, 'handleError') }), retry());
    }
    #selfInject(token) {
        return this.#injector.get(token, undefined, { self: true, optional: true });
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "16.0.0", ngImport: i0, type: WrappedControlSuperclass, deps: [], target: i0.ɵɵFactoryTarget.Directive }); }
    static { this.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "14.0.0", version: "16.0.0", type: WrappedControlSuperclass, usesInheritance: true, ngImport: i0 }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "16.0.0", ngImport: i0, type: WrappedControlSuperclass, decorators: [{
            type: Directive
        }], ctorParameters: function () { return []; } });

/**
 * This class is deprecated because its `control` implementation is not type safe. Transition to a type safe alternative by directly inheriting from {@linkcode WrappedControlSuperclass} and setting up your control with the typing and initial value you need. See the documentation on that class for an example.
 *
 * @deprecated
 */
class WrappedFormControlSuperclass extends WrappedControlSuperclass {
    constructor() {
        super(...arguments);
        this.control = new FormControl();
    }
}

/**
 * A helper to lazy-load Angular services that are not associated with a lazy route. For example, you can split snack bar code into a separate bundle like this:
 *
 * ```ts
 * // snack-bar-bundle.ts
 * export const snackBarBundle = {
 *   tokenMap: { MatSnackBar },
 *   providers: [importProvidersFrom(MatSnackBarModule)],
 * };
 * export type SnackBarBundle = typeof snackBarBundle;
 * export default snackBarBundle;
 *
 * // snack-bar-loader-token.ts
 * export const snackBarLoaderToken = new InjectionToken('snack bar loader', {
 *   factory: (): LazyLoader<SnackBarBundle> =>
 *     new LazyLoader(import('./snack-bar-bundle'))
 * });
 *
 * // my-component.ts
 * @Component({
 *   template: `<button (click)=showSnackBar()>Click me</button>`
 * })
 * class MyComponent {
 *   #snackBarLoader = inject(snackBarLoaderToken);
 *
 *   async showSnackBar(): Promise<void> {
 *     const matSnackBar = await this.#snackBarLoader.inject('MatSnackBar');
 *     matSnackBar.open('You clicked the button');
 *   }
 * }
 * ```
 *
 * Angular tests don't play well with lazy loading, so be sure to check out {@link provideEagerLoading}.
 */
class LazyLoader {
    #resolvedBundle;
    /**
     * This must be created in the context of an environment injector (e.g. during construction of a service or component).
     */
    constructor(bundleImport) {
        const injector = inject(EnvironmentInjector);
        this.#resolvedBundle = bundleImport.then((module) => ({
            injector: createEnvironmentInjector(module.default.providers ?? [], injector),
            tokenMap: module.default.tokenMap,
        }));
    }
    /**
     * Gets an injectable (e.g. a service) from the lazy bundle.
     */
    async inject(key) {
        const { injector, tokenMap } = await this.#resolvedBundle;
        return injector.get(tokenMap[key]);
    }
    /**
     * Gets a provider token from the lazy bundle. This can be useful e.g. when creating a component dynamically, such as for a Material dialog:
     *
     * ```ts
     * // my-dialog.ts
     * @Component({
     *   template: `
     *     <mat-dialog-content>This is a dialog</mat-dialog-content>
     *     <mat-dialog-actions>
     *       <button mat-button mat-dialog-close>OK</button>
     *     </mat-dialog-actions>
     *   `,
     *   standalone: true,
     *   imports: [MatButtonModule, MatDialogModule],
     * })
     * export class MyDialogComponent {}
     *
     * // dialog-bundle.ts
     * export const dialogBundle = {
     *   tokenMap: { MatDialog, MyDialogComponent },
     *   providers: [importProvidersFrom(MatDialogModule)],
     * };
     * export type DialogBundle = typeof dialogBundle;
     * export default dialogBundle;
     *
     * // dialog-loader-token.ts
     * export const dialogLoaderToken = new InjectionToken('dialog loader', {
     *   factory: () => LazyLoader<DialogBundle> =>
     *     new LazyLoader(import('./dialog-bundle'))
     * });
     *
     * // wherever you want to open the dialog
     * const dialogLoader = inject(dialogLoaderToken);
     * const matDialog = await dialogLoader.inject('MatDialog');
     * const myDialogComponent = await dialogLoader.getToken('MyDialogComponent');
     * matDialog.open(myDialogComponent);
     * ```
     */
    async getToken(key) {
        const { tokenMap } = await this.#resolvedBundle;
        return tokenMap[key];
    }
}

/**
 * Use when configuring tests for code that uses a {@link LazyLoader}.
 *
 * Angular tests that use {@link waitForAsync} or {@link fakeAsync} depend on Zone.js to know when asynchronous actions are pending and when they complete. Zone is not aware of the dynamic imports used for lazy loading, so you can use this helper during test setup to work around the issue.
 *
 * ```ts
 * // if you have followed the pattern from LazyLoader docs and created
 * // both a bundle and token, import the bundle directly into your test file
 * import snackBarBundle from './snack-bar-bundle';
 * import { snackBarLoaderToken } from './snack-bar-loader-token';
 *
 * // add to the constructor of AngularContext or ComponentContext
 * const ctx = new AngularContext({
 *   providers: [provideEagerLoading(snackBarLoaderToken, snackBarBundle)],
 * });
 *
 * // or if using TestBed directly, add to configureTestingModule
 * TestBed.configureTestingModule({
 *   providers: [provideEagerLoading(snackBarLoaderToken, snackBarBundle)],
 * });
 * ```
 */
function provideEagerLoading(token, bundle) {
    return {
        provide: token,
        useFactory: () => new LazyLoader(Promise.resolve({ default: bundle })),
    };
}

/*
 * Public API Surface of ng-core
 */

/**
 * Generated bundle index. Do not edit.
 */

export { DirectiveSuperclass, FormComponentSuperclass, InjectableSuperclass, LazyLoader, WrappedControlSuperclass, WrappedFormControlSuperclass, mixInInjectableSuperclass, provideEagerLoading, provideValueAccessor };
//# sourceMappingURL=ng-core.mjs.map
